#!/usr/bin/env python
# -*- coding: utf8 -*-

#-----------------------------------------
# Name:         mm2mm.py
# Author:       Pascal Peter
# Copyright:    (C) 2012 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Email:        pascal.peter at free.fr
#-----------------------------------------


################################################################################
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# -------------------------------------------------------------------------------
# Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le
# modifier conformément aux dispositions de la Licence Publique Générale GNU,
# telle que publiée par la Free Software Foundation ; version 3 de la licence,
# ou encore toute version ultérieure.
# 
# Ce programme est distribué dans l'espoir qu'il sera utile,
# mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION
# ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail,
# voir la Licence Publique Générale GNU.
# 
# Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même
# temps que ce programme ; si ce n'est pas le cas, voir <http://www.gnu.org/licenses/>.
################################################################################

"""
The code is designed to work with Python 2.x (>=2.7) and Python 3.x.
Le code est fait pour fonctionner avec Python 2.x (>=2.7) et avec Python 3.x.

http://drichard.org/mindmaps/
http://framindmap.org/framindmap.html

mm2mm.py test.json
python /home/pascal/Documents/Programmation/mm2mm/mm2mm.py test.json
python3 /home/pascal/Documents/Programmation/mm2mm/mm2mm.py test.json
python3 /home/pascal/Documents/Programmation/mm2mm/mm2mm.py LANG=fr test.json
python3 /home/pascal/Documents/Programmation/mm2mm/mm2mm.py LANG=fr

"""

# import useful modules:
# importation des modules utiles :
from __future__ import division, print_function
import sys
import os
import json
from xml.dom.minidom import Document

# recovery path:
# récupération du chemin :
HERE = os.path.dirname(os.path.abspath(__file__))
# adding HERE to the path:
# ajout du chemin au path :
sys.path.insert(0, HERE)
# it starts in the correct folder:
# on démarre dans le bon dossier :
os.chdir(HERE)

# some useful variables:
# quelques variables utiles :
SEP = os.sep
PYTHONVERSION = sys.version_info[0] * 10 + sys.version_info[1]
LANG = 'en'
RATIO = 2

def main():
    """
    Checks the arguments and starts the conversion.
    --------------------------------------------------------
    Vérifie les arguments, puis lance la conversion.
    """
    jsonFileName, mmFileName = '', ''
    # json and mm files search in arguments:
    # recherche des fichiers json et mm dans les arguments :
    for arg in sys.argv:
        extension = arg.split('.')[-1]
        if extension == 'json':
            jsonFileName = arg
        elif extension == 'mm':
            mmFileName = arg
        if arg.split('=')[0] == 'LANG':
            global LANG
            LANG = arg.split('=')[1]
    # if there is no json file in argument is displayed help:
    # s'il n'y a pas de fichier json en argument, on affiche l'aide :
    if jsonFileName == '':
        doHelp()
        return
    # verification of the existence of the file:
    # vérification de l'existence du fichier :
    if not(os.path.isfile(jsonFileName)):
        if LANG == 'fr':
            message = "Le fichier {0} n'existe pas !"
        else:
            message = 'The file {0} does not exist!'
        print(message.format(jsonFileName))
        return
    # if there is no file argument mm, using the filename json:
    # s'il n'y a pas de fichier mm en argument, on utilise le nom du fichier json :
    if mmFileName == '':
        mmFileName = jsonFileName.replace('.json', '.mm')
    else:
        # verifying the existence of the path:
        # vérification de l'existence du chemin :
        mmFilePath = SEP.join(mmFileName.split(SEP)[:-1])
        if not(os.path.isdir(mmFilePath)):
            if LANG == 'fr':
                message = "Le dossier {0} n'existe pas !"
            else:
                message = 'The directory {0} does not exist!'
            print(message.format(mmFilePath))
            return
    # you can start the conversion:
    # on peut lancer la conversion :
    doConvert(jsonFileName, mmFileName)


def doHelp():
    """
    Displays explanations.
    --------------------------------------------------------
    Affiche les explications.
    Il n'y a aucun accent dans le texte affiché pour éviter les problèmes d'encodage.
    (pas le temps de faire des tests de débogage)
    """
    if LANG == 'fr':
        message = ( '\n'
                    '##################################\n'
                    '     mindmaps -> FreePlane\n'
                    '##################################\n'
                    '\n'
                    'Pour convertir un fichier *.json (produit par mindmaps)\n'
                    'en fichier *.mm (lisible par FreePlane ou FreeMind)\n'
                    '\n'
                    'UTILISATION :\n'
                    '   mm2mm.py fileName.json [fileName.mm] [LANG=xx]\n'
                    '\n'
                    'ARGUMENTS :\n'
                    '   fileName.json : obligatoire\n'
                    '       Nom du fichier a traiter.\n'
                    '       Faire preceder du chemin si pas dans le meme dossier\n'
                    '       que mm2mm.py.\n'
                    '   fileName.mm : facultatif\n'
                    '       Nom du fichier a creer.\n'
                    '       Si omis, le fichier portera le meme nom que le fichier json\n'
                    '       et sera cree dans le meme dossier.\n'
                    '       Faire preceder du chemin si pas dans le meme dossier\n'
                    '       que fileName.json.\n'
                    '   LANG=xx : facultatif\n'
                    '       Pour afficher les messages dans la langue indiquee.\n'
                    "       La langue par défaut est l'Anglais.\n"
                    "       Seul LANG=fr est gere pour l'instant.\n"
                    '\n'
                    '')
    else:
        message = ( '\n'
                    '##################################\n'
                    '     mindmaps -> FreePlane\n'
                    '##################################\n'
                    '\n'
                   'To convert a file *.json (produced by mindmaps)\n'
                    'to *.mm (readable with FreePlane or FreeMind)\n'
                    '\n'
                    'USAGE:\n'
                    '   mm2mm.py fileName.json [fileName.mm] [LANG=xx]\n'
                    '\n'
                    'ARGUMENTS:\n'
                    '   fileName.json : required\n'
                    '       Name of file to be processed.\n'
                    '       To precede the path if not in the same folder\n'
                    '       as mm2mm.py.\n'
                    '   fileName.mm : optional\n'
                    '       Name of file to create.\n'
                    '       If omitted, the file will have the same name as the file json\n'
                    '       and will be created in the same folder.\n'
                    '       To precede the path if not in the same folder\n'
                    '       as fileName.json.\n'
                    '   LANG=xx : optional\n'
                    '       To view the messages in the specified language.\n'
                    '       The default language is English.\n'
                    '       Only LANG=fr gere is now.\n'
                    '\n'
                    '')
    print(message)


def doConvert(jsonFileName, mmFileName):
    """
    The conversion process.
    Reads the json file and creates the mm.
    --------------------------------------------------------
    La procédure de conversion.
    Lit le fichier json et crée le fichier mm.
    """
    # it opens and reads the json:
    # on ouvre et lit le fichier json :
    if PYTHONVERSION >= 30:
        jsonFile = open(jsonFileName, newline='', encoding='utf-8-sig')
    else:
        jsonFile = open(jsonFileName, 'rb')
    jsonContent = json.load(jsonFile)
    jsonFile.close()
    # we create the XML document:
    # on crée le document xml :
    domDocument = Document()
    freemindmap = domDocument.createElement("map")
    domDocument.appendChild(freemindmap)
    freemindmap.setAttribute("version", "0.9.0")
    # nodes in writing DOMDocument:
    # écriture des noeuds dans domDocument :
    mindmap = Mindmap(jsonContent, domDocument, freemindmap)
    # recovery xml text (deleting the first line):
    # récupération du xml en texte (on supprime la première ligne) :
    mmText = domDocument.toprettyxml(indent="  ")
    mmText = '\n'.join(mmText.split('\n')[1:])
    # a final correction for multiple lines of text:
    # une dernière correction pour les textes sur plusieurs lignes :
    mmText = mmText.replace('&amp;#xa;', '&#xa;')
    # is recorded in the file mm:
    # on enregistre dans le fichier mm :
    if PYTHONVERSION >= 30:
        mmFile = open(mmFileName, 'w', newline='', encoding='utf-8')
        mmFile.writelines(mmText)
    else:
        mmFile = open(mmFileName, "wb")
        try:
            mmText = mmText.encode('utf8')
        except:
            if not(isinstance(mmText, str)):
                try:
                    mmText = str(mmText)
                except:
                    print('ERROR:', type(mmText))
        mmFile.writelines(mmText)
    mmFile.close()


class Mindmap:
    """
    Mindmap object.
    Reads the dictionary obtained from json file (mindmapDict)
    and creates nodes of xml document (DOMDocument).
    freemindmap is the starting node.
    --------------------------------------------------------
    Objet Mindmap.
    Lit le dictionnaire obtenu depuis le fichier json (mindmapDict)
    et crée les noeuds du document xml (domDocument).
    freemindmap est le noeud de départ.
    """
    def __init__(self, mindmapDict, domDocument, freemindmap):
        # not everything is used:
        # tout n'est pas utilisé :
        self.id = mindmapDict['id']
        self.title = mindmapDict['title']
        self.autosave = mindmapDict['autosave']
        self.dates = mindmapDict['dates']
        self.dimensions = mindmapDict['dimensions']
        self.mindmap = mindmapDict['mindmap']
        self.pos = (0, 0)
        self.createNodes(domDocument, freemindmap)

    def createNodes(self, domDocument, freemindmap):
        for nodeRoot in self.mindmap:
            nodeDict = self.mindmap[nodeRoot]
            mindmapNode = MindmapNode(nodeDict, domDocument, freemindmap)


class MindmapNode:
    """
    MindmapNode object.
    A map node (mindmap).
    This is where the translations are handled between json and mm.
    --------------------------------------------------------
    Objet MindmapNode.
    Un noeud de la carte (mindmap).
    C'est là que sont gérées les traductions entre json et mm.
    """
    def __init__(self, nodeDict, domDocument, parentNode, parentPos=(0, 0)):

        newNode = domDocument.createElement("node")
        parentNode.appendChild(newNode)

        # node text:
        # texte du noeud :
        text = nodeDict['text']['caption']
        # correction for multiple lines of text:
        # une correction pour les textes sur plusieurs lignes :
        text = text.replace('\n', '&#xa;')
        newNode.setAttribute("TEXT", text)
        # text color:
        # couleur du texte :
        newNode.setAttribute("COLOR", nodeDict['text']['font']['color'])

        # font:
        # police d'écriture :
        font = domDocument.createElement("font")
        newNode.appendChild(font)
        # size:
        # taille :
        font.setAttribute("SIZE", '{0}'.format(nodeDict['text']['font']['size']))
        # bold:
        # gras :
        if nodeDict['text']['font']['weight'] == 'bold':
            font.setAttribute("BOLD", "true")
        # italic:
        # italique :
        if nodeDict['text']['font']['style'] == 'italic':
            font.setAttribute("ITALIC", "true")

        # decoration (strikethrough or underlined):
        # décoration (barré ou souligné) :
        if nodeDict['text']['font']['decoration'] in ('underline', 'line-through'):
            richcontent = domDocument.createElement("richcontent")
            newNode.appendChild(richcontent)
            richcontent.setAttribute("TYPE", "NODE")
            html = domDocument.createElement("html")
            richcontent.appendChild(html)
            head = domDocument.createElement("head")
            html.appendChild(head)
            body = domDocument.createElement("body")
            html.appendChild(body)
            p = domDocument.createElement("p")
            body.appendChild(p)
            if nodeDict['text']['font']['decoration'] == 'underline':
                decoration = domDocument.createElement("u")
            elif nodeDict['text']['font']['decoration'] == 'line-through':
                decoration = domDocument.createElement("strike")
            p.appendChild(decoration)
            theText = domDocument.createTextNode(text)
            decoration.appendChild(theText)

        # not in json:
        # pas dans json :
        #font.setAttribute("NAME", "SansSerif")

        # edge:
        # ligne :
        edge = domDocument.createElement("edge")
        newNode.appendChild(edge)
        # size:
        # taille :
        edge.setAttribute("WIDTH", "4")
        # color:
        # couleur :
        edge.setAttribute("COLOR", nodeDict['branchColor'])

        x = nodeDict['offset']['x']
        y = nodeDict['offset']['y']
        if parentPos == (0, 0):
            if x > 0:
                newNode.setAttribute("POSITION", "right")
            else:
                newNode.setAttribute("POSITION", "left")
        if x > 0:
            pos = (parentPos[0] + x // RATIO, parentPos[1] + y // (3 * RATIO))
        else:
            pos = (parentPos[0] - x // RATIO, parentPos[1] + y // (3 * RATIO))
        #newNode.setAttribute("HGAP", "{0}".format(pos[0]))
        if parentPos == (0, 0):
            newNode.setAttribute("HGAP", "{0}".format(pos[0]))
            newNode.setAttribute("VSHIFT", "{0}".format(pos[1]))


        """
        "foldChildren":false,
        """

        # Finally, it creates children:
        # enfin on crée les enfants :
        for childNodeDict in nodeDict['children']:
            mindmapNode = MindmapNode(childNodeDict, domDocument, newNode, pos)




if __name__ == '__main__':
    main()

